/* --8<--8<--8<--8<--
 *
 * Copyright (C) 2012 Smithsonian Astrophysical Observatory
 *
 * This file is part of blur
 *
 * blur is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * -->8-->8-->8-->8-- */

#include <tparam/tparam.h>
#include <tracefct/tracefct.h>
#include <tracefct/exiterrvals.h>

#include <suplib/debug.h>
#include <suplib/str.h>

#include "main.h"

static TokListToken blurfuncs[] =
{
  {"gauss1d-fwhm",	FUNC_gauss1D_fwhm},
  {"gauss1d-stddev",	FUNC_gauss1D_dev},
  {"rect-uni",		FUNC_rect_uni},
};
static TokList BlurFuncs = GenTokList(blurfuncs);

void
gparam(int argc, char *argv[])
{
  paramfile pf;
  char dummy[NAME_LENGTH+1];

  tf_enter("gparam");

  pf = tparamopen(NULL, argv, argc, "r");

  /* get debug flags */
  tpgetstr(pf, "debug", debug, NAME_LENGTH, 0);
  debug_init(debug);

  c.help =  tpgetb(pf, "help");
  c.version = tpgetb(pf, "version");


  /* input files */
  tpgetstr(pf, "input", c.input, NAME_LENGTH, 1);

  /* output file */
  tpgetstr(pf, "output", c.output, NAME_LENGTH, 1);

  c.seed1 = tpgeti(pf, "seed1");
  if (c.seed1 < 1 || c.seed1 > 2147483562)
    tf_exit(ExitERR_illegalArg, "seed1 must have range [1,2147483562]");

  c.seed2 = tpgeti(pf, "seed2");
  if (c.seed2 < 1 || c.seed2 > 2147483398)
    tf_exit(ExitERR_illegalArg, "seed2 must have range [1,2147483398]");

  c.block = tpgeti(pf, "block");
  if (c.block < 0 || c.block > 1048575)
    tf_exit(ExitERR_illegalArg, "block must have range [0,1048575]");

  tpgetstr(pf, "func", dummy, NAME_LENGTH, 1);
  if ((c.blurfunc = tokmatch(dummy, &BlurFuncs)) == -1)
    tf_exit(ExitERR_misc, "unknown blur function `%s'", dummy);

  switch (c.blurfunc)
  {
  case FUNC_gauss1D_fwhm:
    /* get sigma out of FWHM */
    c.std_dev = tpgetd(pf, "fwhm") / 2.354820054504;
    c.blurfunc = FUNC_gauss1D;
    break;

  case FUNC_gauss1D_dev:
    c.std_dev = tpgetd(pf, "stddev");
    c.blurfunc = FUNC_gauss1D;
    break;

  case FUNC_rect_uni:
    c.half_height = tpgetd( pf, "height" ) / 2.0;
    c.half_width = tpgetd( pf, "width" ) / 2.0;
    break;

  default:
    tf_assert( 0 );
  }

  c.scale = tpgetd( pf, "scale" );

  paramclose(pf);
  tf_leave();
}
